(*
	Vanshil Shah
*)
type action = Raw | Ast | Sast | Codegen

let string_of_action = function
	Raw -> "Raw"
  | Ast -> "Ast"
  | Sast -> "Sast"
  | Codegen -> "Codegen"
let filename = ref ""
let opts = ref Raw

let speclist = [
	("-r", Arg.Unit (fun _ -> opts := Raw), "Raw");
	("-a", Arg.Unit (fun _ -> opts := Ast), "Ast");
	("-s", Arg.Unit (fun _ -> opts := Sast), "Sast");
    ("-c", Arg.Unit (fun _ -> opts := Codegen), "Codegen");
    ("-f", Arg.Set_string filename, "Setting filename");
  ]

let usage = "usage: " ^ Sys.argv.(0) ^ " [-r] [-a] [-s] [-c] [-f <Filename>]"

let () =
	Arg.parse
		speclist
		(fun x -> raise (Arg.Bad ("Bad Argument : " ^ x)))
		usage;

	let in_channel = open_in !filename in
	let lexbuf = Lexing.from_channel in_channel in
	let program = Parser.program Scanner.token lexbuf in
	match !opts with
		Raw -> print_string (Ast.program_s program) 
	|	Ast -> let listing = Ast.program_s program in 
				print_string listing
	|	Sast -> (try 
					ignore (Analyzer.infer_prog program);
					print_endline "Semantic analysis complete"
				with
					Failure(x) -> print_endline x)
    |   Codegen ->
            let main_out = open_out "main.cpp" in
			let a_program = try 
				Analyzer.infer_prog program 
			with
				Failure(x) -> 
					print_endline ("Semantic analysis failed: " ^ x);
					([],[])
			in
			if a_program <> ([],[])
			then
				let listing = Codegen.string_of_prog a_program
                in output_string main_out listing 
